@@ -5,7 +5,10 @@ EVE_CATEGORY_ID_STRUCTURE = 65
 
 EVE_GROUP_ID_CONTROL_TOWER = 365
 EVE_GROUP_ID_FUEL_BLOCK = 1136
+EVE_GROUP_ID_ENGINERING_COMPLEX = 1404
 EVE_GROUP_ID_REFINERY = 1406
+EVE_GROUP_ID_JUMP_GATE = 1408
+EVE_GROUP_ID_CITADEL = 1657
 
 EVE_TYPE_ID_POCO = 2233
 EVE_TYPE_ID_TCU = 32226
